      subroutine convmix(itime)
      
C     handles all the calculations related to convective mixing
C     Petra Seibert, Bernd C. Krueger, Feb 2001
c     nested grids included, Bernd C. Krueger, May 2001

      include 'includepar'
      include 'includecom'


      integer ier, igr, igrold, ipart, itime, ix, j, inest
      integer jy, kpart, ktop, ngrid
      integer igrid(maxpart), ipoint(maxpart), igridn(maxpart,maxnests)
C igrid(maxpart)            horizontal grid position of each particle
C igridn(maxpart,maxnests)  dto. for nested grids
C ipoint(maxpart)           pointer to access particles according to grid position

c     monitoring variables
      real sumall,sumconv

      logical lconv
      real x, y, xtn,ytn, ztold

      integer itage,nage


C =========================================================================
C CHANGES by A. Stohl:
C No convection calculation for particles that are just being started

C remove enquiry for polarstereographic grids

C At the end of subroutine various lines to calculate fluxes across
C control surfaces
C =========================================================================

      lconv = .true. ! has to go to a global swich later

c     initialization of igrid and igridn
      do ipart = 1,numpart
         igrid(ipart) = -1
         do inest = 1,numbnests
            igridn(ipart,inest) = -1
         enddo
      enddo
C this is a pseudo grid number indicating particles that are outside
C the part of the grid under consideration (e.g. particles near the poles
c or particles in other nests)

C calculate horiz. grid positions of particles so that they can be sorted
C************************************************************************

      do 20 ipart=1,numpart
C CHANGE: do not use particles that are just being started
         if (itra1(ipart).ne.itime) then
C        do not consider particles that are (yet) not part of simulation
            goto 20
         endif
         x = xtra1(ipart)
         y = ytra1(ipart)
        
c code taken from advance.f [begin]

C Determine which nesting level to be used
***********************************************************

         ngrid=0
         do 22 j=numbnests,1,-1
           if ( x.gt.xln(j) .and. x.lt.xrn(j) .and.
     +          y.gt.yln(j) .and. y.lt.yrn(j) ) then
                  ngrid=j
                  goto 23
           endif
 22        continue
 23      continue
      
C Determine nested grid coordinates
***********************************

C-cv for sake of simplicity and also because convection is not so
C-cv important in polar regions, do not perform convective mixing
C-cv in the polarstereographic grid regions (hard coded |lat|>75deg
C-cv in present code)
         if (ngrid.gt.0) then
C-cv  nested grids
            xtn=(x-xln(ngrid))*xresoln(ngrid)
            ytn=(y-yln(ngrid))*yresoln(ngrid)
            ix=int(xtn)
            jy=int(ytn)
            igr = 1 + jy*nxn(ngrid) + ix
            igridn(ipart,ngrid) = igr
         elseif(ngrid.eq.0) then
c     mother grid
            ix=int(x)
            jy=int(y)
            igr = 1 + jy*nx + ix
            igrid(ipart) = igr
         endif

 20   continue
c--------------------------------------------------------------------------
c     initialize monitoring variables
      sumall = 0.
      sumconv = 0.
c--------------------------------------------------------------------------

C sort particles according to horizontal position in mother domain:
C calculate index vector IPOINT

      call ipsort (igrid, numpart, ipoint, 1, ier)
      if (ier .ne. 0) then
         print*, 'error ocurred in IPSORT'
         stop
      endif

C Now visit all grid columns where particles are present
C by going through the sorted particles

      igrold = -1
      do 50 kpart=1,numpart

         ipart = ipoint(kpart)
         igr = igrid(ipart)
         if (igr .eq. -1) goto 50
         sumall = sumall + 1
         if (igr. ne. igrold) then
C        we are in a new grid column; calculate translocation matrix
            jy = (igr-1)/nx
            ix = igr - jy*nx - 1
C! check this formulation!          
            call calcmatrix(ix,jy,lconv)
            igrold = igr
            ktop = 0
         endif
        
C       treat particle only if column has convection
         if (lconv .eqv. .true.) then                
C         assign new vertical position to particle

            ztold=ztra1(ipart)
            call redist(ipart,ktop,ix,jy)
            if (ipart.le.0) then
               sumconv = sumconv+1
               ipart=(-1)*ipart

C Calculate the gross fluxes across layer interfaces
****************************************************

               itage=abs(itra1(ipart)-itramem(ipart))
               do 36 nage=1,nageclass
                  if (itage.lt.lage(nage)) goto 37
 36            continue
 37            continue

               if ((iflux.eq.1).and.(nage.le.nageclass))
     $              call calcfluxes(nage,ipart,
     $              xtra1(ipart),ytra1(ipart),ztold)

            endif
         endif
 50   continue
c--------------------------------------------------------------------------

C sort particles according to horizontal position in nests:
      do 70 inest=1,numbnests
C calculate index vector IPOINT
         do ipart=1,numpart
            igrid(ipart) = igridn(ipart,inest)
         enddo
         call ipsort (igrid, numpart, ipoint, 1, ier)
         if (ier .ne. 0) then
            print*, 'error ocurred in IPSORT'
            stop
         endif

C Now visit all grid columns where particles are present
C by going through the sorted particles

         igrold = -1
         do 60 kpart=1,numpart

            ipart = ipoint(kpart)
            igr = igrid(ipart)
            if (igr .eq. -1) goto 60
            sumall = sumall + 1
            if (igr. ne. igrold) then
C         we are in a new grid column; calculate translocation matrix
               jy = (igr-1)/nxn(inest)
               ix = igr - jy*nxn(inest) - 1
C! check this formulation!          
               call calcmatrix_nests(inest,ix,jy,lconv)
               igrold = igr
               ktop = 0
            endif
        
C       treat particle only if column has convection
            if (lconv .eqv. .true.) then                
C         assign new vertical position to particle

               ztold=ztra1(ipart)
               call redist_nests(inest,ipart,ktop,ix,jy)
               if (ipart.le.0) then
                  sumconv = sumconv+1
                  ipart=(-1)*ipart

C Calculate the gross fluxes across layer interfaces
****************************************************

                  itage=abs(itra1(ipart)-itramem(ipart))
                  do 46 nage=1,nageclass
                     if (itage.lt.lage(nage)) goto 47
 46               continue
 47               continue

                  if ((iflux.eq.1).and.(nage.le.nageclass))
     $                 call calcfluxes(nage,ipart,
     $                 xtra1(ipart),ytra1(ipart),ztold)
               endif
            endif
 60      continue
 70   continue
c--------------------------------------------------------------------------
C-cv  write(933,*)'fraction of particles under convection',
C-cv &  sumconv/(sumall+0.001) ! 933

      return
      end
