      subroutine outgridvolume()
********************************************************************************
*                                                                              *
*  This routine calculates, for each grid cell of the output grid, the         *
*  volume, the surface area, and the areas of the northward and eastward       *
*  facing surfaces.                                                            *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     7 August 2002                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
*                                                                              *
* area               surface area of all output grid cells                     *
* areae              eastward facing wall area of all output grid cells        *
* arean              northward facing wall area of all output grid cells       *
* volume             volumes of all output grid cells                          *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      
      integer ix,jy,kz
      real ylat,gridarea,ylatp,ylatm,hzone,cosfact,cosfactm,cosfactp


C Compute surface area and volume of each grid cell: area, volume;
C and the areas of the northward and eastward facing walls: areae, arean
************************************************************************

      do 10 jy=0,numygrid-1
        ylat=outlat0+(float(jy)+0.5)*dyout
        ylatp=ylat+0.5*dyout
        ylatm=ylat-0.5*dyout
        if ((ylatm.lt.0).and.(ylatp.gt.0.)) then
          hzone=dyout*r_earth*pi180
        else

C Calculate area of grid cell with formula M=2*pi*R*h*dx/360,
C see Netz, Formeln der Mathematik, 5. Auflage (1983), p.90
*************************************************************

          cosfact=cos(ylat*pi180)*r_earth
          cosfactp=cos(ylatp*pi180)*r_earth
          cosfactm=cos(ylatm*pi180)*r_earth
          if (cosfactp.lt.cosfactm) then
            hzone=sqrt(r_earth**2-cosfactp**2)-
     +      sqrt(r_earth**2-cosfactm**2)
          else
            hzone=sqrt(r_earth**2-cosfactm**2)-
     +      sqrt(r_earth**2-cosfactp**2)
          endif
        endif

C Surface are of a grid cell at a latitude ylat
***********************************************

        gridarea=2.*pi*r_earth*hzone*dxout/360.

        do 10 ix=0,numxgrid-1
          area(ix,jy)=gridarea

C Volume = area x box height
****************************

          volume(ix,jy,1)=area(ix,jy)*outheight(1)
          areae(ix,jy,1)=dyout*r_earth*pi180*outheight(1)
          arean(ix,jy,1)=cos(ylat*pi180)*dxout*r_earth*pi180*
     +    outheight(1)
          do 10 kz=2,numzgrid
            areae(ix,jy,kz)=dyout*r_earth*pi180*
     +      (outheight(kz)-outheight(kz-1))
            arean(ix,jy,kz)=cos(ylat*pi180)*dxout*r_earth*pi180*
     +      (outheight(kz)-outheight(kz-1))
10          volume(ix,jy,kz)=area(ix,jy)*(outheight(kz)-outheight(kz-1))
 

      end
