       subroutine redist_nests (nest,ipart,ktop,ix,jy)
       
C Do the redistribution of particles due to convection
C This subroutine is called for each particle which is assigned
C a new vertical position randomly, based on the convective redistribution
C matrix

C Petra Seibert, Feb 2001, Apr 2001, May 2001, Jan 2002
c version for nested grid, Bernd C. Krueger, May 2001
C Andreas Frank, Nov 2002

      include 'includepar'
      include 'includecom'
      include 'includeconv'

      real const
      parameter (const=r_air/ga) ! from vertransform.f
      integer nest,ipart, ktop
      integer iseed, ix, jy, k, kz, levnew, levold, n
      real uvzlev(nuvzmax), ctlv(nconvlevmax), ctlvsum
      real dlev, pint, pold, rn, sumtrac, tv, tvold
      real ew,ran3, ztold
      real tv1, tv2
      save iseed, uvzlev, ctlv
      data iseed /701/

C ipart   ... number of particle to be treated

C     assign time indices - note that calculation of convection must
C     be done forward in the true time both in fwd or bwd calculations!
      if(ldirect.eq.1)then
         n = memind(1)
      else
         n=memind(2)
      endif

C  determine vertical grid position of particle in the eta system
C----------------------------------------------------------------

      if (ktop .le. 1) then 
C       initialise calculation of height agl of eta levels (uvzlev)

C code taken from verttransform [begin]
C   [note that this piece of code is now used already 3 times - 
C    maybe it could be put in subroutine/functions later!]
        tvold=tt2n(ix,jy,1,n,nest)*(1.+0.378*ew(td2n(ix,jy,1,n,nest))/
     &                                   psn(ix,jy,1,n,nest))
        pold=psn(ix,jy,1,n,nest)
        uvzlev(1)=0.
        pint = phlev(2)*100.
C       determine next virtual temperatures        
        tv1 = tlev(1)*(1.+0.608*qlev(1))
        tv2 = tlev(2)*(1.+0.608*qlev(2))
C       interpolate virtual temperature to half-level
        tv = tv1 + (tv2-tv1)*(plev(1)-phlev(2))/(plev(1)-plev(2))
        if (abs(tv-tvold).gt.0.2) then
          uvzlev(2) = uvzlev(1) + const*log(pold/pint)*
     &                 (tv-tvold)/log(tv/tvold)
        else
          uvzlev(2) = uvzlev(1)+const*log(pold/pint)*tv
        endif
        tvold=tv
        tv1=tv2
        pold=pint       
        
        ktop = 2
        
C-cv integrate profile (calculation of height agl of eta layers) as required
C---------------------------------------------------------------------------
        do 10 kz = 3, nconvlev+1
C         note that variables defined in calcmatrix.f (plev,tlev,qlev)
C         start at the first real ECMWF model level whereas kz and
C         thus uvzlev(kz) starts at the surface. unzlev is defined at the
C         half-levels (between the tlev, ctralev etc. values !)
C         Thus, uvzlev(kz) is the lower boundary of the ctralev(kz) cell.
          k = kz-1
          pint = phlev(kz)*100.
C         determine next virtual temperatures        
          tv2 = tlev(kz)*(1.+0.608*qlev(kz))
C         interpolate virtual temperature to half-level
          tv = tv1 + (tv2-tv1)*(plev(kz-1)-phlev(kz))/
     $         (plev(kz-1)-plev(kz))
          if (abs(tv-tvold).gt.0.2) then
C-cv note that the 0.2 deg threshold should be reduced for good accuracy
C-cv present formulation may cause errors ~metres in upper atmosphere            
            uvzlev(kz) = uvzlev(kz-1) + const*log(pold/pint)*
     &                 (tv-tvold)/log(tv/tvold)
          else
            uvzlev(kz) = uvzlev(kz-1)+const*log(pold/pint)*tv
          endif
          tvold=tv
          tv1=tv2
          pold=pint
10      continue
      endif ! C-cv

      ztold = ztra1(ipart)
C     find old particle grid position 
      do 20 kz = 1, nconvlev 
        if (uvzlev(kz) .ge. ztold ) then
          levold = kz-1
C         this is the index of the corresponding tracer species 
cc          dz = (ztold-uvzlev(kz-1)) / (uvzlev(kz)-uvzlev(kz-1))
          tvold=tv
          pold=pint
          goto 30
        endif 
        tvold=tv
        pold=pint
20    continue

C Particle is above the potentially convective domain. Skip it.
      goto 90

30    continue

C Interpolate ctralev for exact level of origin
C Keep the division by ctm0 immediately after ctralev for numerical reasons.   
CC note that we do not interpolate now. it seems not to be
CC good numerically. 
C note that transposed ctralev matrix to be used for backward simulation
C PS / Jan 2002

      ctlvsum = 0.
      do 32 k = 1,nconvlev
        if(ldirect .eq. 1) then 
          ctlv(k) = ctralev(k,levold)*dp(k) / ctm0
        else
          ctlv(k) = ctralev(levold,k)*dp(k) / ctm0
        endif
        ctlvsum = ctlvsum + ctlv(k)
cc         ctlv(k) = ( (1.-dz)*ctralev(k,levold)*dp(k)
cc     &                 + dz*ctralev(k,levold+1)*dp(k) )/ ctm0 
cc        ctlv(k) = ( ((1.-dz)*ctralev(k,levold) + dz*ctralev(k,levold+1))
cc     &           / ctm0 )*dp(levold)
cc        if (dz.lt.0.5) then
cc       ctlv(k) = ( ctralev(k,levold) / ctm0 )*dp(k)
cc        else
cc        ctlv(k) = ( ctralev(k,levold+1) / ctm0 )*dp(k)
cc        endif
32    continue

C  Choose a random number and find corresponding level of destination
C--------------------------------------------------------------------
C  Random numbers to be evenly distributed in [0,1]

C     the following line is an ad-hoc bug fix. Reason why this condition
C     can occur has to be found out and properly corrected later.

      if (ctlvsum .lt. 1.e-20) goto 90
      
      rn = ran3(iseed)
      sumtrac = 0.
      
C now we build up the cumulative distribution of the redistributed
C tracer profile. Total sum of all levels is 1. Thus, we can choose
C sumtrac=rn to find the destination level of the particle.

      do 35 k = 1,nconvlev
        sumtrac = sumtrac + ctlv(k)/ctlvsum
        if (sumtrac .ge. rn) then
          levnew = k
C the following if-clause is a bug fix to avoid division by zero or
C underflow/overflow problems for the case of a ration of two small numbers
C in this case, the particle is assigned to the centre of the cell (dlev=.5)
C [fix introduced May 28, 2001, by P. Seibert]
          if ((ctlv(k)/ctlvsum) .gt. 1.e-20) then
            dlev = (sumtrac-rn) / (ctlv(k)/ctlvsum)
          else
            dlev = 0.5
          endif
          goto 40
        endif
35    continue

       levnew = nconvlev
       dlev = 0.

40    continue

C now assign new position to particle
C------------------------------------
C note that we could do it in a more sophisticated way
C using dz, and pressure instead of z, but that must
C wait until later

      if (levnew.le.nconvlev) then
        if (levnew.eq.levold .and. 
     &        ctralev(levnew,levold)/ctm0.gt.0.9999) then
C         avoid artifical spreading in areas without convection
          ztra1(ipart) = ztold
        else
          ztra1(ipart) = uvzlev(levnew)+
     &       (1.-dlev)*(uvzlev(levnew+1)-uvzlev(levnew))
          ipart=-ipart
        endif
      else
        goto 90
      endif

C Maximum altitude .5 meter below uppermost model level
********************************************************

 90   continue
      if (ztra1(abs(ipart)).gt.(height(nz)-0.5))
     +     ztra1(abs(ipart))=height(nz)-0.5

      return
      end
