      subroutine writeheader()
********************************************************************************
*                                                                              *
*  This routine produces a file header containing basic information on the     *
*  settings of the FLEXPART run.                                               *
*  The header file is essential and must be read in by any postprocessing      *
*  program before reading in the output data.                                  *
*                                                                              *
*     Author: A. Stohl                                                         *
*                                                                              *
*     7 August 2002                                                            *
*                                                                              *
********************************************************************************
*                                                                              *
* Variables:                                                                   *
*                                                                              *
* xlon                   longitude                                             *
* xl                     model x coordinate                                    *
* ylat                   latitude                                              *
* yl                     model y coordinate                                    *
*                                                                              *
*                                                                              *
********************************************************************************

      include 'includepar'
      include 'includecom'
      
      integer jjjjmmdd,ihmmss,i,ix,jy,j,iix,jjy,ixp,jyp,i1,j1,ngrid
      real xlon,ylat,xp1,yp1,xp2,yp2,xl,yl,ddx,ddy,rddx,rddy,p1,p2,p3,p4
      real xtn,ytn,oroh


**************************************************************************
C First, determine average height of model topography in output grid cells
**************************************************************************

C Loop over all output grid cells
*********************************

      do 20 jjy=0,numygrid-1
        do 20 iix=0,numxgrid-1
          oroh=0.

C Take 100 samples of the topography in every grid cell
*******************************************************

          do 22 j1=1,10
            ylat=outlat0+(float(jjy)+float(j1)/10.-0.05)*dyout
            yl=(ylat-ylat0)/dy
            do 22 i1=1,10
              xlon=outlon0+(float(iix)+float(i1)/10.-0.05)*dxout
              xl=(xlon-xlon0)/dx

C Determine the nest we are in
******************************

              ngrid=0
              do 42 j=numbnests,1,-1
                if ((xl.gt.xln(j)).and.(xl.lt.xrn(j)).and.
     +          (yl.gt.yln(j)).and.(yl.lt.yrn(j))) then
                  ngrid=j
                  goto 43
                endif
42              continue
43            continue

C Determine (nested) grid coordinates and auxiliary parameters used for interpolation
*************************************************************************************

              if (ngrid.gt.0) then
                xtn=(xl-xln(ngrid))*xresoln(ngrid)
                ytn=(yl-yln(ngrid))*yresoln(ngrid)
                ix=int(xtn)
                jy=int(ytn)
                ddy=ytn-float(jy)
                ddx=xtn-float(ix)
              else
                ix=int(xl)
                jy=int(yl)
                ddy=yl-float(jy)
                ddx=xl-float(ix)
              endif
              ixp=ix+1
              jyp=jy+1
              rddx=1.-ddx
              rddy=1.-ddy
              p1=rddx*rddy
              p2=ddx*rddy
              p3=rddx*ddy
              p4=ddx*ddy

              if (ngrid.gt.0) then
                oroh=oroh+p1*oron(ix ,jy ,ngrid)
     +                  + p2*oron(ixp,jy ,ngrid)
     +                  + p3*oron(ix ,jyp,ngrid)
     +                  + p4*oron(ixp,jyp,ngrid)
              else
                oroh=oroh+p1*oro(ix ,jy)
     +                  + p2*oro(ixp,jy)
     +                  + p3*oro(ix ,jyp)
     +                  + p4*oro(ixp,jyp)
              endif
22            continue

C Divide by the number of samples taken
***************************************

          oroout(iix,jjy)=oroh/100.    
20        continue


*************************
C Open header output file
*************************

      open(unitheader,file=path(2)(1:len(2))//'header',
     +form='unformatted',err=998)


C Write the header information
******************************

      if (ldirect.eq.1) then
        write(unitheader) ibdate,ibtime,'FLEXPART V5.0'
      else
        write(unitheader) iedate,ietime,'FLEXPART V5.0'
      endif

C Write info on output interval, averaging time, sampling time
**************************************************************

      write(unitheader) loutstep,loutaver,loutsample

C Write information on output grid setup
****************************************

      write(unitheader) outlon0,outlat0,numxgrid,numygrid,
     +dxout,dyout
      write(unitheader) numzgrid,(outheight(i),i=1,numzgrid)

      call caldate(bdate,jjjjmmdd,ihmmss)
      write(unitheader) jjjjmmdd,ihmmss

C Write number of species, and name for each species (+extra name for depositions)
C Indicate the dimension of the fields (i.e., 1 for deposition fields, numzgrid for
C concentration fields
***********************************************************************************

      write(unitheader) 3*nspec
      do 12 i=1,nspec
        write(unitheader) 1,'WD_'//species(i)(1:7)
        write(unitheader) 1,'DD_'//species(i)(1:7)
12      write(unitheader) numzgrid,species(i)

C Write information on release points: total number, then for each point:
C start, end, coordinates, # of particles, name, mass
*************************************************************************

      write(unitheader) numpoint
      do 13 i=1,numpoint
        write(unitheader) ireleasestart(i),ireleaseend(i)
        xp1=xpoint1(i)*dx+xlon0
        yp1=ypoint1(i)*dy+ylat0
        xp2=xpoint2(i)*dx+xlon0
        yp2=ypoint2(i)*dy+ylat0
        write(unitheader) xp1,yp1,xp2,yp2,zpoint1(i),zpoint2(i)
        write(unitheader) npart(i),1
        write(unitheader) compoint(i)
        do 13 j=1,nspec
          write(unitheader) xmass(i,j)
          write(unitheader) xmass(i,j)
13        write(unitheader) xmass(i,j)

C Write information on some model switches
******************************************

      write(unitheader) method,lsubgrid,lconvection

C Write age class information
*****************************

      write(unitheader) nageclass,(lage(i),i=1,nageclass)


C Write topography to output file
*********************************

      do 30 ix=0,numxgrid-1
30      write(unitheader) (oroout(ix,jy),jy=0,numygrid-1)
      close(unitheader)

      return


998   write(*,*) ' #### FLEXPART MODEL ERROR!   THE FILE         #### '
      write(*,*) ' #### '//path(2)(1:len(2))//'header'//' #### '
      write(*,*) ' #### CANNOT BE OPENED. IF A FILE WITH THIS    #### '
      write(*,*) ' #### NAME ALREADY EXISTS, DELETE IT AND START #### '
      write(*,*) ' #### THE PROGRAM AGAIN.                       #### '
      stop

      end
